@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,8 +33,6 @@
 #include "gc/z/zStat.hpp"
 #include "logging/log.hpp"
 
-#include <limits>
-
 ZDirector* ZDirector::_director;
 
 constexpr double one_in_1000 = 3.290527;
@@ -145,11 +143,11 @@ static double select_young_gc_workers(const ZDirectorStats& stats, double serial
   return gc_workers;
 }
 
-ZDriverRequest rule_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
-                                                  double serial_gc_time_passed,
-                                                  double parallel_gc_time_passed,
-                                                  bool conservative_alloc_rate,
-                                                  size_t capacity) {
+static ZDriverRequest rule_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
+                                                         double serial_gc_time_passed,
+                                                         double parallel_gc_time_passed,
+                                                         bool conservative_alloc_rate,
+                                                         size_t capacity) {
   if (!stats._old_stats._cycle._is_time_trustable) {
     // Rule disabled
     return ZDriverRequest(GCCause::_no_gc, ZYoungGCThreads, 0);
@@ -216,9 +214,9 @@ ZDriverRequest rule_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
   return ZDriverRequest(GCCause::_z_allocation_rate, actual_gc_workers, 0);
 }
 
-ZDriverRequest rule_soft_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
-                                                       double serial_gc_time_passed,
-                                                       double parallel_gc_time_passed) {
+static ZDriverRequest rule_soft_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
+                                                              double serial_gc_time_passed,
+                                                              double parallel_gc_time_passed) {
     return rule_minor_allocation_rate_dynamic(stats,
                                               0.0 /* serial_gc_time_passed */,
                                               0.0 /* parallel_gc_time_passed */,
@@ -226,9 +224,9 @@ ZDriverRequest rule_soft_minor_allocation_rate_dynamic(const ZDirectorStats& sta
                                               stats._heap._soft_max_heap_size /* capacity */);
 }
 
-ZDriverRequest rule_semi_hard_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
-                                                            double serial_gc_time_passed,
-                                                            double parallel_gc_time_passed) {
+static ZDriverRequest rule_semi_hard_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
+                                                                   double serial_gc_time_passed,
+                                                                   double parallel_gc_time_passed) {
   return rule_minor_allocation_rate_dynamic(stats,
                                             0.0 /* serial_gc_time_passed */,
                                             0.0 /* parallel_gc_time_passed */,
@@ -236,9 +234,9 @@ ZDriverRequest rule_semi_hard_minor_allocation_rate_dynamic(const ZDirectorStats
                                             ZHeap::heap()->max_capacity() /* capacity */);
 }
 
-ZDriverRequest rule_hard_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
-                                                       double serial_gc_time_passed,
-                                                       double parallel_gc_time_passed) {
+static ZDriverRequest rule_hard_minor_allocation_rate_dynamic(const ZDirectorStats& stats,
+                                                              double serial_gc_time_passed,
+                                                              double parallel_gc_time_passed) {
   return rule_minor_allocation_rate_dynamic(stats,
                                             0.0 /* serial_gc_time_passed */,
                                             0.0 /* parallel_gc_time_passed */,
@@ -455,22 +453,16 @@ static double calculate_extra_young_gc_time(const ZDirectorStats& stats) {
   // relocation headroom into account to avoid in-place relocation.
   const size_t old_used = stats._old_stats._general._used;
   const size_t old_live = stats._old_stats._stat_heap._live_at_mark_end;
-  const double old_garbage = double(old_used - old_live);
+  const size_t old_garbage = old_used - old_live;
 
   const double young_gc_time = gc_time(stats._young_stats);
 
   // Calculate how much memory young collections are predicted to free.
-  const double reclaimed_per_young_gc = stats._young_stats._stat_heap._reclaimed_avg;
+  const size_t reclaimed_per_young_gc = stats._young_stats._stat_heap._reclaimed_avg;
 
   // Calculate current YC time and predicted YC time after an old collection.
-  const double current_young_gc_time_per_bytes_freed = young_gc_time / reclaimed_per_young_gc;
-  const double potential_young_gc_time_per_bytes_freed = young_gc_time / (reclaimed_per_young_gc + old_garbage);
-
-  if (current_young_gc_time_per_bytes_freed == std::numeric_limits<double>::infinity()) {
-    // Young collection's are not reclaiming any memory. Return infinity as a signal
-    // to trigger an old collection, regardless of the amount of old garbage.
-    return std::numeric_limits<double>::infinity();
-  }
+  const double current_young_gc_time_per_bytes_freed = double(young_gc_time) / double(reclaimed_per_young_gc);
+  const double potential_young_gc_time_per_bytes_freed = double(young_gc_time) / double(reclaimed_per_young_gc + old_garbage);
 
   // Calculate extra time per young collection inflicted by *not* doing an
   // old collection that frees up memory in the old generation.
@@ -491,12 +483,12 @@ static bool rule_major_allocation_rate(const ZDirectorStats& stats) {
   const double young_gc_time = gc_time(stats._young_stats);
 
   // Calculate how much memory collections are predicted to free.
-  const double reclaimed_per_young_gc = stats._young_stats._stat_heap._reclaimed_avg;
-  const double reclaimed_per_old_gc = stats._old_stats._stat_heap._reclaimed_avg;
+  const size_t reclaimed_per_young_gc = stats._young_stats._stat_heap._reclaimed_avg;
+  const size_t reclaimed_per_old_gc = stats._old_stats._stat_heap._reclaimed_avg;
 
   // Calculate the GC cost for each reclaimed byte
-  const double current_young_gc_time_per_bytes_freed = young_gc_time / reclaimed_per_young_gc;
-  const double current_old_gc_time_per_bytes_freed = old_gc_time / reclaimed_per_old_gc;
+  const double current_young_gc_time_per_bytes_freed = double(young_gc_time) / double(reclaimed_per_young_gc);
+  const double current_old_gc_time_per_bytes_freed = double(old_gc_time) / double(reclaimed_per_old_gc);
 
   // Calculate extra time per young collection inflicted by *not* doing an
   // old collection that frees up memory in the old generation.
@@ -532,29 +524,12 @@ static bool rule_major_allocation_rate(const ZDirectorStats& stats) {
 }
 
 static double calculate_young_to_old_worker_ratio(const ZDirectorStats& stats) {
-  if (!stats._old_stats._cycle._is_time_trustable) {
-    return 1.0;
-  }
-
   const double young_gc_time = gc_time(stats._young_stats);
   const double old_gc_time = gc_time(stats._old_stats);
-  const double reclaimed_per_young_gc = stats._young_stats._stat_heap._reclaimed_avg;
-  const double reclaimed_per_old_gc = stats._old_stats._stat_heap._reclaimed_avg;
-  const double current_young_bytes_freed_per_gc_time = reclaimed_per_young_gc / young_gc_time;
-  const double current_old_bytes_freed_per_gc_time = reclaimed_per_old_gc / old_gc_time;
-
-  if (current_young_bytes_freed_per_gc_time == 0.0) {
-    if (current_old_bytes_freed_per_gc_time == 0.0) {
-      // Neither young nor old collections have reclaimed any memory.
-      // Give them equal priority.
-      return 1.0;
-    }
-
-    // Only old collections have reclaimed memory.
-    // Prioritize old.
-    return ZOldGCThreads;
-  }
-
+  const size_t reclaimed_per_young_gc = stats._young_stats._stat_heap._reclaimed_avg;
+  const size_t reclaimed_per_old_gc = stats._old_stats._stat_heap._reclaimed_avg;
+  const double current_young_bytes_freed_per_gc_time = double(reclaimed_per_young_gc) / double(young_gc_time);
+  const double current_old_bytes_freed_per_gc_time = double(reclaimed_per_old_gc) / double(old_gc_time);
   const double old_vs_young_efficiency_ratio = current_old_bytes_freed_per_gc_time / current_young_bytes_freed_per_gc_time;
 
   return old_vs_young_efficiency_ratio;
