@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -42,8 +42,8 @@ class CompositeFunctor {
   Func2* _g;
  public:
   CompositeFunctor(Func1* f, Func2* g) : _f(f), _g(g) {
-    assert(f != NULL, "invariant");
-    assert(g != NULL, "invariant");
+    assert(f != nullptr, "invariant");
+    assert(g != nullptr, "invariant");
   }
   bool operator()(T const& value) {
     return (*_f)(value) && (*_g)(value);
@@ -63,11 +63,11 @@ class JfrArtifactCallbackHost : public JfrArtifactClosure {
  public:
   JfrArtifactCallbackHost(JfrArtifactClosure** subsystem_callback_loc, Callback* callback) :
           _subsystem_callback_loc(subsystem_callback_loc), _callback(callback) {
-    assert(*_subsystem_callback_loc == NULL, "Subsystem callback should not be set yet");
+    assert(*_subsystem_callback_loc == nullptr, "Subsystem callback should not be set yet");
     *_subsystem_callback_loc = this;
   }
   ~JfrArtifactCallbackHost() {
-    *_subsystem_callback_loc = NULL;
+    *_subsystem_callback_loc = nullptr;
   }
   void do_artifact(const void* artifact) {
     (*_callback)(reinterpret_cast<T const&>(artifact));
@@ -80,8 +80,9 @@ class KlassToFieldEnvelope {
  public:
   KlassToFieldEnvelope(Letter* letter) : _letter(letter) {}
   bool operator()(const Klass* klass) {
+    assert(IS_SERIALIZED(klass), "invariant");
     typename FieldSelector::TypePtr t = FieldSelector::select(klass);
-    return t != NULL ? (*_letter)(t) : true;
+    return t != nullptr ? (*_letter)(t) : true;
   }
 };
 
@@ -91,6 +92,8 @@ class ClearArtifact {
   bool operator()(T const& value) {
     CLEAR_SERIALIZED(value);
     assert(IS_NOT_SERIALIZED(value), "invariant");
+    assert(IS_NOT_LEAKP(value), "invariant");
+    assert(IS_NOT_TRANSIENT(value), "invariant");
     SET_PREVIOUS_EPOCH_CLEARED_BIT(value);
     CLEAR_PREVIOUS_EPOCH_METHOD_AND_CLASS(value);
     return true;
@@ -103,42 +106,22 @@ class ClearArtifact<const Method*> {
   bool operator()(const Method* method) {
     assert(METHOD_FLAG_USED_PREVIOUS_EPOCH(method), "invariant");
     CLEAR_SERIALIZED_METHOD(method);
-    assert(METHOD_NOT_SERIALIZED(method), "invariant");
+    assert(METHOD_IS_NOT_SERIALIZED(method), "invariant");
+    assert(METHOD_IS_NOT_LEAKP(method), "invariant");
+    assert(METHOD_IS_NOT_TRANSIENT(method), "invariant");
     SET_PREVIOUS_EPOCH_METHOD_CLEARED_BIT(method);
     CLEAR_PREVIOUS_EPOCH_METHOD_FLAG(method);
     return true;
   }
 };
 
-template <typename T>
-class SerializePredicate {
-  bool _class_unload;
- public:
-  SerializePredicate(bool class_unload) : _class_unload(class_unload) {}
-  bool operator()(T const& value) {
-    assert(value != NULL, "invariant");
-    return _class_unload ? true : IS_NOT_SERIALIZED(value);
-  }
-};
-
-template <>
-class SerializePredicate<const Method*> {
-  bool _class_unload;
- public:
-  SerializePredicate(bool class_unload) : _class_unload(class_unload) {}
-  bool operator()(const Method* method) {
-    assert(method != NULL, "invariant");
-    return _class_unload ? true : METHOD_NOT_SERIALIZED(method);
-  }
-};
-
 template <typename T, bool leakp>
 class SymbolPredicate {
   bool _class_unload;
  public:
   SymbolPredicate(bool class_unload) : _class_unload(class_unload) {}
   bool operator()(T const& value) {
-    assert(value != NULL, "invariant");
+    assert(value != nullptr, "invariant");
     if (_class_unload) {
       return leakp ? value->is_leakp() : value->is_unloading();
     }
@@ -146,11 +129,23 @@ class SymbolPredicate {
   }
 };
 
+class KlassUsedPredicate {
+  bool _current_epoch;
+ public:
+  KlassUsedPredicate(bool current_epoch) : _current_epoch(current_epoch) {}
+  bool operator()(const Klass* klass) {
+    return _current_epoch ? USED_THIS_EPOCH(klass) : USED_PREVIOUS_EPOCH(klass);
+  }
+};
+
 class MethodUsedPredicate {
   bool _current_epoch;
 public:
   MethodUsedPredicate(bool current_epoch) : _current_epoch(current_epoch) {}
   bool operator()(const Klass* klass) {
+    if (!klass->is_instance_klass()) {
+      return false;
+    }
     return _current_epoch ? METHOD_USED_THIS_EPOCH(klass) : METHOD_USED_PREVIOUS_EPOCH(klass);
   }
 };
@@ -162,9 +157,9 @@ class MethodFlagPredicate {
   MethodFlagPredicate(bool current_epoch) : _current_epoch(current_epoch) {}
   bool operator()(const Method* method) {
     if (_current_epoch) {
-      return leakp ? IS_METHOD_LEAKP_USED(method) : METHOD_FLAG_USED_THIS_EPOCH(method);
+      return leakp ? METHOD_IS_LEAKP(method) : METHOD_FLAG_USED_THIS_EPOCH(method);
     }
-    return leakp ? IS_METHOD_LEAKP_USED(method) : METHOD_FLAG_USED_PREVIOUS_EPOCH(method);
+    return leakp ? METHOD_IS_LEAKP(method) : METHOD_FLAG_USED_PREVIOUS_EPOCH(method);
   }
 };
 
@@ -182,8 +177,8 @@ class LeakPredicate<const Method*> {
  public:
   LeakPredicate(bool class_unload) {}
   bool operator()(const Method* method) {
-    assert(method != NULL, "invariant");
-    return IS_METHOD_LEAKP_USED(method);
+    assert(method != nullptr, "invariant");
+    return METHOD_IS_LEAKP(method);
   }
 };
 
@@ -198,7 +193,7 @@ class LeakPredicate<const Method*> {
  * during the write process itself.
  *
  * It can also provide opportunities for caching, as the ideal should
- * be to reduce the amount of iterations neccessary for locating artifacts
+ * be to reduce the amount of iterations necessary for locating artifacts
  * in the respective VM subsystems.
  */
 class JfrArtifactSet : public JfrCHeapObj {
@@ -206,7 +201,9 @@ class JfrArtifactSet : public JfrCHeapObj {
   JfrSymbolTable* _symbol_table;
   GrowableArray<const Klass*>* _klass_list;
   GrowableArray<const Klass*>* _klass_loader_set;
+  GrowableArray<const Klass*>* _klass_loader_leakp_set;
   size_t _total_count;
+  bool _class_unload;
 
  public:
   JfrArtifactSet(bool class_unload);
@@ -231,18 +228,9 @@ class JfrArtifactSet : public JfrCHeapObj {
   int entries() const;
   size_t total_count() const;
   void register_klass(const Klass* k);
-  bool should_do_loader_klass(const Klass* k);
+  bool should_do_cld_klass(const Klass* k, bool leakp);
   void increment_checkpoint_id();
 
-  template <typename Functor>
-  void iterate_klasses(Functor& functor) const {
-    for (int i = 0; i < _klass_list->length(); ++i) {
-      if (!functor(_klass_list->at(i))) {
-        break;
-      }
-    }
-  }
-
   template <typename T>
   void iterate_symbols(T& functor) {
     _symbol_table->iterate_symbols(functor);
@@ -258,6 +246,24 @@ class JfrArtifactSet : public JfrCHeapObj {
     _total_count += writer.count();
   }
 
+  template <typename Functor>
+  void iterate_klasses(Functor& functor) const {
+    if (iterate(functor, _klass_list)) {
+      iterate(functor, _klass_loader_set);
+    }
+  }
+
+ private:
+  template <typename Functor>
+  bool iterate(Functor& functor, GrowableArray<const Klass*>* list) const {
+    assert(list != nullptr, "invariant");
+    for (int i = 0; i < list->length(); ++i) {
+      if (!functor(list->at(i))) {
+        return false;
+      }
+    }
+    return true;
+  }
 };
 
 class KlassArtifactRegistrator {
@@ -266,11 +272,11 @@ class KlassArtifactRegistrator {
  public:
   KlassArtifactRegistrator(JfrArtifactSet* artifacts) :
     _artifacts(artifacts) {
-    assert(_artifacts != NULL, "invariant");
+    assert(_artifacts != nullptr, "invariant");
   }
 
   bool operator()(const Klass* klass) {
-    assert(klass != NULL, "invariant");
+    assert(klass != nullptr, "invariant");
     _artifacts->register_klass(klass);
     return true;
   }
