@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,16 +26,14 @@
 #include "classfile/vmClasses.hpp"
 #include "classfile/vmSymbols.hpp"
 #include "code/codeCache.hpp"
-#include "code/compiledMethod.inline.hpp"
 #include "code/compiledIC.hpp"
-#include "code/icBuffer.hpp"
 #include "code/nmethod.hpp"
 #include "code/pcDesc.hpp"
 #include "code/scopeDesc.hpp"
 #include "code/vtableStubs.hpp"
 #include "compiler/compileBroker.hpp"
 #include "compiler/oopMap.hpp"
-#include "gc/g1/heapRegion.hpp"
+#include "gc/g1/g1HeapRegion.hpp"
 #include "gc/shared/barrierSet.hpp"
 #include "gc/shared/collectedHeap.hpp"
 #include "gc/shared/gcLocker.hpp"
@@ -85,7 +83,7 @@
 //  To force FullGCALot inside a runtime function, add the following two lines
 //
 //  Universe::release_fullgc_alot_dummy();
-//  MarkSweep::invoke(0, "Debugging");
+//  Universe::heap()->collect();
 //
 // At command line specify the parameters: -XX:+FullGCALot -XX:FullGCALotStart=100000000
 
@@ -93,22 +91,28 @@
 
 
 // Compiled code entry points
-address OptoRuntime::_new_instance_Java                           = NULL;
-address OptoRuntime::_new_array_Java                              = NULL;
-address OptoRuntime::_new_array_nozero_Java                       = NULL;
-address OptoRuntime::_multianewarray2_Java                        = NULL;
-address OptoRuntime::_multianewarray3_Java                        = NULL;
-address OptoRuntime::_multianewarray4_Java                        = NULL;
-address OptoRuntime::_multianewarray5_Java                        = NULL;
-address OptoRuntime::_multianewarrayN_Java                        = NULL;
-address OptoRuntime::_vtable_must_compile_Java                    = NULL;
-address OptoRuntime::_complete_monitor_locking_Java               = NULL;
-address OptoRuntime::_monitor_notify_Java                         = NULL;
-address OptoRuntime::_monitor_notifyAll_Java                      = NULL;
-address OptoRuntime::_rethrow_Java                                = NULL;
+address OptoRuntime::_new_instance_Java                           = nullptr;
+address OptoRuntime::_new_array_Java                              = nullptr;
+address OptoRuntime::_new_array_nozero_Java                       = nullptr;
+address OptoRuntime::_multianewarray2_Java                        = nullptr;
+address OptoRuntime::_multianewarray3_Java                        = nullptr;
+address OptoRuntime::_multianewarray4_Java                        = nullptr;
+address OptoRuntime::_multianewarray5_Java                        = nullptr;
+address OptoRuntime::_multianewarrayN_Java                        = nullptr;
+address OptoRuntime::_vtable_must_compile_Java                    = nullptr;
+address OptoRuntime::_complete_monitor_locking_Java               = nullptr;
+address OptoRuntime::_monitor_notify_Java                         = nullptr;
+address OptoRuntime::_monitor_notifyAll_Java                      = nullptr;
+address OptoRuntime::_rethrow_Java                                = nullptr;
 
-address OptoRuntime::_slow_arraycopy_Java                         = NULL;
-address OptoRuntime::_register_finalizer_Java                     = NULL;
+address OptoRuntime::_slow_arraycopy_Java                         = nullptr;
+address OptoRuntime::_register_finalizer_Java                     = nullptr;
+#if INCLUDE_JVMTI
+address OptoRuntime::_notify_jvmti_vthread_start                  = nullptr;
+address OptoRuntime::_notify_jvmti_vthread_end                    = nullptr;
+address OptoRuntime::_notify_jvmti_vthread_mount                  = nullptr;
+address OptoRuntime::_notify_jvmti_vthread_unmount                = nullptr;
+#endif
 
 ExceptionBlob* OptoRuntime::_exception_blob;
 
@@ -130,7 +134,7 @@ static bool check_compiled_frame(JavaThread* thread) {
 
 #define gen(env, var, type_func_gen, c_func, fancy_jump, pass_tls, return_pc) \
   var = generate_stub(env, type_func_gen, CAST_FROM_FN_PTR(address, c_func), #var, fancy_jump, pass_tls, return_pc); \
-  if (var == NULL) { return false; }
+  if (var == nullptr) { return false; }
 
 bool OptoRuntime::generate(ciEnv* env) {
 
@@ -148,6 +152,12 @@ bool OptoRuntime::generate(ciEnv* env) {
   gen(env, _multianewarray4_Java           , multianewarray4_Type         , multianewarray4_C               ,    0 , true, false);
   gen(env, _multianewarray5_Java           , multianewarray5_Type         , multianewarray5_C               ,    0 , true, false);
   gen(env, _multianewarrayN_Java           , multianewarrayN_Type         , multianewarrayN_C               ,    0 , true, false);
+#if INCLUDE_JVMTI
+  gen(env, _notify_jvmti_vthread_start     , notify_jvmti_vthread_Type    , SharedRuntime::notify_jvmti_vthread_start, 0, true, false);
+  gen(env, _notify_jvmti_vthread_end       , notify_jvmti_vthread_Type    , SharedRuntime::notify_jvmti_vthread_end,   0, true, false);
+  gen(env, _notify_jvmti_vthread_mount     , notify_jvmti_vthread_Type    , SharedRuntime::notify_jvmti_vthread_mount, 0, true, false);
+  gen(env, _notify_jvmti_vthread_unmount   , notify_jvmti_vthread_Type    , SharedRuntime::notify_jvmti_vthread_unmount, 0, true, false);
+#endif
   gen(env, _complete_monitor_locking_Java  , complete_monitor_enter_Type  , SharedRuntime::complete_monitor_locking_C, 0, false, false);
   gen(env, _monitor_notify_Java            , monitor_notify_Type          , monitor_notify_C                ,    0 , false, false);
   gen(env, _monitor_notifyAll_Java         , monitor_notify_Type          , monitor_notifyAll_C             ,    0 , false, false);
@@ -181,7 +191,7 @@ const char* OptoRuntime::stub_name(address entry) {
 #ifndef PRODUCT
   CodeBlob* cb = CodeCache::find_blob(entry);
   RuntimeStub* rs =(RuntimeStub *)cb;
-  assert(rs != NULL && rs->is_runtime_stub(), "not a runtime stub");
+  assert(rs != nullptr && rs->is_runtime_stub(), "not a runtime stub");
   return rs->name();
 #else
   // Fast implementation for product mode (maybe it should be inlined too)
@@ -303,19 +313,22 @@ JRT_BLOCK_ENTRY(void, OptoRuntime::new_array_nozero_C(Klass* array_type, int len
   SharedRuntime::on_slowpath_allocation_exit(current);
 
   oop result = current->vm_result();
-  if ((len > 0) && (result != NULL) &&
+  if ((len > 0) && (result != nullptr) &&
       is_deoptimized_caller_frame(current)) {
     // Zero array here if the caller is deoptimized.
     const size_t size = TypeArrayKlass::cast(array_type)->oop_size(result);
     BasicType elem_type = TypeArrayKlass::cast(array_type)->element_type();
-    const size_t hs = arrayOopDesc::header_size(elem_type);
-    // Align to next 8 bytes to avoid trashing arrays's length.
-    const size_t aligned_hs = align_object_offset(hs);
+    size_t hs_bytes = arrayOopDesc::base_offset_in_bytes(elem_type);
+    assert(is_aligned(hs_bytes, BytesPerInt), "must be 4 byte aligned");
     HeapWord* obj = cast_from_oop<HeapWord*>(result);
-    if (aligned_hs > hs) {
-      Copy::zero_to_words(obj+hs, aligned_hs-hs);
+    if (!is_aligned(hs_bytes, BytesPerLong)) {
+      *reinterpret_cast<jint*>(reinterpret_cast<char*>(obj) + hs_bytes) = 0;
+      hs_bytes += BytesPerInt;
     }
+
     // Optimized zeroing.
+    assert(is_aligned(hs_bytes, BytesPerLong), "must be 8-byte aligned");
+    const size_t aligned_hs = hs_bytes / BytesPerLong;
     Copy::fill_to_aligned_words(obj+aligned_hs, size-aligned_hs);
   }
 
@@ -466,6 +479,22 @@ const TypeFunc *OptoRuntime::new_instance_Type() {
   return TypeFunc::make(domain, range);
 }
 
+#if INCLUDE_JVMTI
+const TypeFunc *OptoRuntime::notify_jvmti_vthread_Type() {
+  // create input type (domain)
+  const Type **fields = TypeTuple::fields(2);
+  fields[TypeFunc::Parms+0] = TypeInstPtr::NOTNULL; // VirtualThread oop
+  fields[TypeFunc::Parms+1] = TypeInt::BOOL;        // jboolean
+  const TypeTuple *domain = TypeTuple::make(TypeFunc::Parms+2,fields);
+
+  // no result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms+0] = nullptr; // void
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
+
+  return TypeFunc::make(domain,range);
+}
+#endif
 
 const TypeFunc *OptoRuntime::athrow_Type() {
   // create input type (domain)
@@ -609,12 +638,12 @@ const TypeFunc *OptoRuntime::monitor_notify_Type() {
 const TypeFunc* OptoRuntime::flush_windows_Type() {
   // create input type (domain)
   const Type** fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL; // void
+  fields[TypeFunc::Parms+0] = nullptr; // void
   const TypeTuple *domain = TypeTuple::make(TypeFunc::Parms, fields);
 
   // create result type
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL; // void
+  fields[TypeFunc::Parms+0] = nullptr; // void
   const TypeTuple *range = TypeTuple::make(TypeFunc::Parms, fields);
 
   return TypeFunc::make(domain, range);
@@ -742,6 +771,29 @@ const TypeFunc* OptoRuntime::void_void_Type() {
  }
 
 
+// Takes as parameters:
+// void *dest
+// long size
+// uchar byte
+const TypeFunc* OptoRuntime::make_setmemory_Type() {
+  // create input type (domain)
+  int argcnt = NOT_LP64(3) LP64_ONLY(4);
+  const Type** fields = TypeTuple::fields(argcnt);
+  int argp = TypeFunc::Parms;
+  fields[argp++] = TypePtr::NOTNULL;        // dest
+  fields[argp++] = TypeX_X;                 // size
+  LP64_ONLY(fields[argp++] = Type::HALF);   // size
+  fields[argp++] = TypeInt::UBYTE;          // bytevalue
+  assert(argp == TypeFunc::Parms+argcnt, "correct decoding");
+  const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
+
+  // no result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms+0] = nullptr; // void
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
+  return TypeFunc::make(domain, range);
+}
+
 // arraycopy stub variations:
 enum ArrayCopyType {
   ac_fast,                      // void(ptr, ptr, size_t)
@@ -781,7 +833,7 @@ static const TypeFunc* make_arraycopy_Type(ArrayCopyType act) {
   int retcnt = (act == ac_checkcast || act == ac_generic ? 1 : 0);
   fields = TypeTuple::fields(1);
   if (retcnt == 0)
-    fields[TypeFunc::Parms+0] = NULL; // void
+    fields[TypeFunc::Parms+0] = nullptr; // void
   else
     fields[TypeFunc::Parms+0] = TypeInt::INT; // status result, if needed
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms+retcnt, fields);
@@ -823,12 +875,55 @@ const TypeFunc* OptoRuntime::array_fill_Type() {
 
   // create result type
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL; // void
+  fields[TypeFunc::Parms+0] = nullptr; // void
   const TypeTuple *range = TypeTuple::make(TypeFunc::Parms, fields);
 
   return TypeFunc::make(domain, range);
 }
 
+const TypeFunc* OptoRuntime::array_partition_Type() {
+  // create input type (domain)
+  int num_args = 7;
+  int argcnt = num_args;
+  const Type** fields = TypeTuple::fields(argcnt);
+  int argp = TypeFunc::Parms;
+  fields[argp++] = TypePtr::NOTNULL;  // array
+  fields[argp++] = TypeInt::INT;      // element type
+  fields[argp++] = TypeInt::INT;      // low
+  fields[argp++] = TypeInt::INT;      // end
+  fields[argp++] = TypePtr::NOTNULL;  // pivot_indices (int array)
+  fields[argp++] = TypeInt::INT;      // indexPivot1
+  fields[argp++] = TypeInt::INT;      // indexPivot2
+  assert(argp == TypeFunc::Parms+argcnt, "correct decoding");
+  const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
+
+  // no result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms+0] = nullptr; // void
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
+  return TypeFunc::make(domain, range);
+}
+
+const TypeFunc* OptoRuntime::array_sort_Type() {
+  // create input type (domain)
+  int num_args      = 4;
+  int argcnt = num_args;
+  const Type** fields = TypeTuple::fields(argcnt);
+  int argp = TypeFunc::Parms;
+  fields[argp++] = TypePtr::NOTNULL;    // array
+  fields[argp++] = TypeInt::INT;    // element type
+  fields[argp++] = TypeInt::INT;    // fromIndex
+  fields[argp++] = TypeInt::INT;    // toIndex
+  assert(argp == TypeFunc::Parms+argcnt, "correct decoding");
+  const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
+
+  // no result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms+0] = nullptr; // void
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
+  return TypeFunc::make(domain, range);
+}
+
 // for aescrypt encrypt/decrypt operations, just three pointers returning void (length is constant)
 const TypeFunc* OptoRuntime::aescrypt_block_Type() {
   // create input type (domain)
@@ -844,7 +939,7 @@ const TypeFunc* OptoRuntime::aescrypt_block_Type() {
 
   // no result type needed
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL; // void
+  fields[TypeFunc::Parms+0] = nullptr; // void
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1018,13 +1113,13 @@ const TypeFunc* OptoRuntime::digestBase_implCompress_Type(bool is_sha3) {
   int argp = TypeFunc::Parms;
   fields[argp++] = TypePtr::NOTNULL; // buf
   fields[argp++] = TypePtr::NOTNULL; // state
-  if (is_sha3) fields[argp++] = TypeInt::INT; // digest_length
+  if (is_sha3) fields[argp++] = TypeInt::INT; // block_size
   assert(argp == TypeFunc::Parms+argcnt, "correct decoding");
   const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
 
   // no result type needed
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL; // void
+  fields[TypeFunc::Parms+0] = nullptr; // void
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1040,7 +1135,7 @@ const TypeFunc* OptoRuntime::digestBase_implCompressMB_Type(bool is_sha3) {
   int argp = TypeFunc::Parms;
   fields[argp++] = TypePtr::NOTNULL; // buf
   fields[argp++] = TypePtr::NOTNULL; // state
-  if (is_sha3) fields[argp++] = TypeInt::INT; // digest_length
+  if (is_sha3) fields[argp++] = TypeInt::INT; // block_size
   fields[argp++] = TypeInt::INT;     // ofs
   fields[argp++] = TypeInt::INT;     // limit
   assert(argp == TypeFunc::Parms+argcnt, "correct decoding");
@@ -1055,7 +1150,7 @@ const TypeFunc* OptoRuntime::digestBase_implCompressMB_Type(bool is_sha3) {
 
 const TypeFunc* OptoRuntime::multiplyToLen_Type() {
   // create input type (domain)
-  int num_args      = 6;
+  int num_args      = 5;
   int argcnt = num_args;
   const Type** fields = TypeTuple::fields(argcnt);
   int argp = TypeFunc::Parms;
@@ -1064,13 +1159,12 @@ const TypeFunc* OptoRuntime::multiplyToLen_Type() {
   fields[argp++] = TypePtr::NOTNULL;    // y
   fields[argp++] = TypeInt::INT;        // ylen
   fields[argp++] = TypePtr::NOTNULL;    // z
-  fields[argp++] = TypeInt::INT;        // zlen
   assert(argp == TypeFunc::Parms+argcnt, "correct decoding");
   const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
 
   // no result type needed
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL;
+  fields[TypeFunc::Parms+0] = nullptr;
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1090,7 +1184,7 @@ const TypeFunc* OptoRuntime::squareToLen_Type() {
 
   // no result type needed
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms+0] = NULL;
+  fields[TypeFunc::Parms+0] = nullptr;
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1178,7 +1272,7 @@ const TypeFunc * OptoRuntime::bigIntegerShift_Type() {
 
   // no result type needed
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms + 0] = NULL;
+  fields[TypeFunc::Parms + 0] = nullptr;
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1218,10 +1312,30 @@ const TypeFunc* OptoRuntime::ghash_processBlocks_Type() {
 
     // result type needed
     fields = TypeTuple::fields(1);
-    fields[TypeFunc::Parms+0] = NULL; // void
+    fields[TypeFunc::Parms+0] = nullptr; // void
     const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
     return TypeFunc::make(domain, range);
 }
+
+// ChaCha20 Block function
+const TypeFunc* OptoRuntime::chacha20Block_Type() {
+    int argcnt = 2;
+
+    const Type** fields = TypeTuple::fields(argcnt);
+    int argp = TypeFunc::Parms;
+    fields[argp++] = TypePtr::NOTNULL;      // state
+    fields[argp++] = TypePtr::NOTNULL;      // result
+
+    assert(argp == TypeFunc::Parms + argcnt, "correct decoding");
+    const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms + argcnt, fields);
+
+    // result type needed
+    fields = TypeTuple::fields(1);
+    fields[TypeFunc::Parms + 0] = TypeInt::INT;     // key stream outlen as int
+    const TypeTuple* range = TypeTuple::make(TypeFunc::Parms + 1, fields);
+    return TypeFunc::make(domain, range);
+}
+
 // Base64 encode function
 const TypeFunc* OptoRuntime::base64_encodeBlock_Type() {
   int argcnt = 6;
@@ -1239,7 +1353,7 @@ const TypeFunc* OptoRuntime::base64_encodeBlock_Type() {
 
   // result type needed
   fields = TypeTuple::fields(1);
-  fields[TypeFunc::Parms + 0] = NULL; // void
+  fields[TypeFunc::Parms + 0] = nullptr; // void
   const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1266,6 +1380,65 @@ const TypeFunc* OptoRuntime::base64_decodeBlock_Type() {
   return TypeFunc::make(domain, range);
 }
 
+// Poly1305 processMultipleBlocks function
+const TypeFunc* OptoRuntime::poly1305_processBlocks_Type() {
+  int argcnt = 4;
+
+  const Type** fields = TypeTuple::fields(argcnt);
+  int argp = TypeFunc::Parms;
+  fields[argp++] = TypePtr::NOTNULL;    // input array
+  fields[argp++] = TypeInt::INT;        // input length
+  fields[argp++] = TypePtr::NOTNULL;    // accumulator array
+  fields[argp++] = TypePtr::NOTNULL;    // r array
+  assert(argp == TypeFunc::Parms + argcnt, "correct decoding");
+  const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
+
+  // result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms + 0] = nullptr; // void
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
+  return TypeFunc::make(domain, range);
+}
+
+// MontgomeryIntegerPolynomialP256 multiply function
+const TypeFunc* OptoRuntime::intpoly_montgomeryMult_P256_Type() {
+  int argcnt = 3;
+
+  const Type** fields = TypeTuple::fields(argcnt);
+  int argp = TypeFunc::Parms;
+  fields[argp++] = TypePtr::NOTNULL;    // a array
+  fields[argp++] = TypePtr::NOTNULL;    // b array
+  fields[argp++] = TypePtr::NOTNULL;    // r(esult) array
+  assert(argp == TypeFunc::Parms + argcnt, "correct decoding");
+  const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
+
+  // result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms + 0] = TypeInt::INT; // carry bits in output
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms+1, fields);
+  return TypeFunc::make(domain, range);
+}
+
+// IntegerPolynomial constant time assignment function
+const TypeFunc* OptoRuntime::intpoly_assign_Type() {
+  int argcnt = 4;
+
+  const Type** fields = TypeTuple::fields(argcnt);
+  int argp = TypeFunc::Parms;
+  fields[argp++] = TypeInt::INT;        // set flag
+  fields[argp++] = TypePtr::NOTNULL;    // a array (result)
+  fields[argp++] = TypePtr::NOTNULL;    // b array (if set is set)
+  fields[argp++] = TypeInt::INT;        // array length
+  assert(argp == TypeFunc::Parms + argcnt, "correct decoding");
+  const TypeTuple* domain = TypeTuple::make(TypeFunc::Parms+argcnt, fields);
+
+  // result type needed
+  fields = TypeTuple::fields(1);
+  fields[TypeFunc::Parms + 0] = NULL; // void
+  const TypeTuple* range = TypeTuple::make(TypeFunc::Parms, fields);
+  return TypeFunc::make(domain, range);
+}
+
 //------------- Interpreter state access for on stack replacement
 const TypeFunc* OptoRuntime::osr_end_Type() {
   // create input type (domain)
@@ -1276,7 +1449,7 @@ const TypeFunc* OptoRuntime::osr_end_Type() {
   // create result type
   fields = TypeTuple::fields(1);
   // fields[TypeFunc::Parms+0] = TypeInstPtr::NOTNULL; // locked oop
-  fields[TypeFunc::Parms+0] = NULL; // void
+  fields[TypeFunc::Parms+0] = nullptr; // void
   const TypeTuple *range = TypeTuple::make(TypeFunc::Parms, fields);
   return TypeFunc::make(domain, range);
 }
@@ -1315,8 +1488,8 @@ JRT_ENTRY_NO_ASYNC(address, OptoRuntime::handle_exception_C_helper(JavaThread* c
   // is only used to pass arguments into the method. Not for general
   // exception handling.  DO NOT CHANGE IT to use pending_exception, since
   // the runtime stubs checks this on exit.
-  assert(current->exception_oop() != NULL, "exception oop is found");
-  address handler_address = NULL;
+  assert(current->exception_oop() != nullptr, "exception oop is found");
+  address handler_address = nullptr;
 
   Handle exception(current, current->exception_oop());
   address pc = current->exception_pc();
@@ -1349,7 +1522,7 @@ JRT_ENTRY_NO_ASYNC(address, OptoRuntime::handle_exception_C_helper(JavaThread* c
   // using rethrow node
 
   nm = CodeCache::find_nmethod(pc);
-  assert(nm != NULL, "No NMethod found");
+  assert(nm != nullptr, "No NMethod found");
   if (nm->is_native_method()) {
     fatal("Native method should not have path to exception handling");
   } else {
@@ -1390,12 +1563,12 @@ JRT_ENTRY_NO_ASYNC(address, OptoRuntime::handle_exception_C_helper(JavaThread* c
     } else {
 
       handler_address =
-        force_unwind ? NULL : nm->handler_for_exception_and_pc(exception, pc);
+        force_unwind ? nullptr : nm->handler_for_exception_and_pc(exception, pc);
 
-      if (handler_address == NULL) {
+      if (handler_address == nullptr) {
         bool recursive_exception = false;
         handler_address = SharedRuntime::compute_compiled_exc_handler(nm, pc, exception, force_unwind, true, recursive_exception);
-        assert (handler_address != NULL, "must have compiled handler");
+        assert (handler_address != nullptr, "must have compiled handler");
         // Update the exception cache only when the unwind was not forced
         // and there didn't happen another exception during the computation of the
         // compiled exception handler. Checking for exception oop equality is not
@@ -1441,8 +1614,8 @@ address OptoRuntime::handle_exception_C(JavaThread* current) {
   SharedRuntime::_find_handler_ctr++;          // find exception handler
 #endif
   debug_only(NoHandleMark __hm;)
-  nmethod* nm = NULL;
-  address handler_address = NULL;
+  nmethod* nm = nullptr;
+  address handler_address = nullptr;
   {
     // Enter the VM
 
@@ -1455,7 +1628,7 @@ address OptoRuntime::handle_exception_C(JavaThread* current) {
   // Now check to see if the handler we are returning is in a now
   // deoptimized frame
 
-  if (nm != NULL) {
+  if (nm != nullptr) {
     RegisterMap map(current,
                     RegisterMap::UpdateMap::skip,
                     RegisterMap::ProcessFrames::skip,
@@ -1494,20 +1667,13 @@ address OptoRuntime::handle_exception_C(JavaThread* current) {
 // *THIS IS NOT RECOMMENDED PROGRAMMING STYLE*
 //
 address OptoRuntime::rethrow_C(oopDesc* exception, JavaThread* thread, address ret_pc) {
-
-  // Enable WXWrite: the function called directly by compiled code.
-  MACOS_AARCH64_ONLY(ThreadWXEnable wx(WXWrite, thread));
-
   // ret_pc will have been loaded from the stack, so for AArch64 will be signed.
-  // This needs authenticating, but to do that here requires the fp of the previous frame.
-  // A better way of doing it would be authenticate in the caller by adding a
-  // AuthPAuthNode and using it in GraphKit::gen_stub. For now, just strip it.
-  AARCH64_PORT_ONLY(ret_pc = pauth_strip_pointer(ret_pc));
+  AARCH64_PORT_ONLY(ret_pc = pauth_strip_verifiable(ret_pc));
 
 #ifndef PRODUCT
   SharedRuntime::_rethrow_ctr++;               // count rethrows
 #endif
-  assert (exception != NULL, "should have thrown a NULLPointerException");
+  assert (exception != nullptr, "should have thrown a NullPointerException");
 #ifdef ASSERT
   if (!(exception->is_a(vmClasses::Throwable_klass()))) {
     // should throw an exception here
@@ -1646,7 +1812,7 @@ JRT_END
 
 //-----------------------------------------------------------------------------
 
-NamedCounter * volatile OptoRuntime::_named_counters = NULL;
+NamedCounter * volatile OptoRuntime::_named_counters = nullptr;
 
 //
 // dump the collected NamedCounters.
@@ -1702,7 +1868,7 @@ NamedCounter* OptoRuntime::new_named_counter(JVMState* youngest_jvms, NamedCount
   stringStream st;
   for (int depth = max_depth; depth >= 1; depth--) {
     JVMState* jvms = youngest_jvms->of_depth(depth);
-    ciMethod* m = jvms->has_method() ? jvms->method() : NULL;
+    ciMethod* m = jvms->has_method() ? jvms->method() : nullptr;
     if (!first) {
       st.print(" ");
     } else {
@@ -1710,7 +1876,7 @@ NamedCounter* OptoRuntime::new_named_counter(JVMState* youngest_jvms, NamedCount
     }
     int bci = jvms->bci();
     if (bci < 0) bci = 0;
-    if (m != NULL) {
+    if (m != nullptr) {
       st.print("%s.%s", m->holder()->name()->as_utf8(), m->name()->as_utf8());
     } else {
       st.print("no method");
@@ -1729,7 +1895,7 @@ NamedCounter* OptoRuntime::new_named_counter(JVMState* youngest_jvms, NamedCount
   // add counters so this is safe.
   NamedCounter* head;
   do {
-    c->set_next(NULL);
+    c->set_next(nullptr);
     head = _named_counters;
     c->set_next(head);
   } while (Atomic::cmpxchg(&_named_counters, head, c) != head);
@@ -1745,9 +1911,8 @@ static void trace_exception(outputStream* st, oop exception_oop, address excepti
   exception_oop->print_value_on(&tempst);
   tempst.print(" in ");
   CodeBlob* blob = CodeCache::find_blob(exception_pc);
-  if (blob->is_compiled()) {
-    CompiledMethod* cm = blob->as_compiled_method_or_null();
-    cm->method()->print_value_on(&tempst);
+  if (blob->is_nmethod()) {
+    blob->as_nmethod()->method()->print_value_on(&tempst);
   } else if (blob->is_runtime_stub()) {
     tempst.print("<runtime-stub>");
   } else {
