@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -82,10 +82,20 @@ void ThreadShadow::clear_pending_nonasync_exception() {
 
 // Implementation of Exceptions
 
-bool Exceptions::special_exception(JavaThread* thread, const char* file, int line, Handle h_exception) {
+bool Exceptions::special_exception(JavaThread* thread, const char* file, int line, Handle h_exception, Symbol* h_name, const char* message) {
+  assert(h_exception.is_null() != (h_name == nullptr), "either exception (" PTR_FORMAT ") or "
+         "symbol (" PTR_FORMAT ") must be non-null but not both", p2i(h_exception()), p2i(h_name));
+
   // bootstrapping check
   if (!Universe::is_fully_initialized()) {
-   vm_exit_during_initialization(h_exception);
+    if (h_exception.not_null()) {
+      vm_exit_during_initialization(h_exception);
+    } else if (h_name == nullptr) {
+      // at least an informative message.
+      vm_exit_during_initialization("Exception", message);
+    } else {
+      vm_exit_during_initialization(h_name, message);
+    }
    ShouldNotReachHere();
   }
 
@@ -94,7 +104,7 @@ bool Exceptions::special_exception(JavaThread* thread, const char* file, int lin
   // to prevent infinite recursion trying to initialize stack overflow without
   // adequate stack space.
   // This can happen with stress testing a large value of StackShadowPages
-  if (h_exception()->klass() == vmClasses::StackOverflowError_klass()) {
+  if (h_exception.not_null() && h_exception()->klass() == vmClasses::StackOverflowError_klass()) {
     InstanceKlass* ik = InstanceKlass::cast(h_exception->klass());
     assert(ik->is_initialized(),
            "need to increase java_thread_min_stack_allowed calculation");
@@ -102,6 +112,16 @@ bool Exceptions::special_exception(JavaThread* thread, const char* file, int lin
 #endif // ASSERT
 
   if (!thread->can_call_java()) {
+    ResourceMark rm(thread);
+    const char* exc_value = h_exception.not_null() ? h_exception->print_value_string() :
+                      h_name != nullptr ? h_name->as_C_string() :
+                      "null";
+    log_info(exceptions)("Thread cannot call Java so instead of throwing exception <%s%s%s> (" PTR_FORMAT ") \n"
+                        "at [%s, line %d]\nfor thread " PTR_FORMAT ",\n"
+                        "throwing pre-allocated exception: %s",
+                        exc_value, message ? ": " : "", message ? message : "",
+                        p2i(h_exception()), file, line, p2i(thread),
+                        Universe::vm_exception()->print_value_string());
     // We do not care what kind of exception we get for a thread which
     // is compiling.  We just install a dummy exception object
     thread->set_pending_exception(Universe::vm_exception(), file, line);
@@ -111,27 +131,6 @@ bool Exceptions::special_exception(JavaThread* thread, const char* file, int lin
   return false;
 }
 
-bool Exceptions::special_exception(JavaThread* thread, const char* file, int line, Symbol* h_name, const char* message) {
-  // bootstrapping check
-  if (!Universe::is_fully_initialized()) {
-    if (h_name == nullptr) {
-      // at least an informative message.
-      vm_exit_during_initialization("Exception", message);
-    } else {
-      vm_exit_during_initialization(h_name, message);
-    }
-    ShouldNotReachHere();
-  }
-
-  if (!thread->can_call_java()) {
-    // We do not care what kind of exception we get for a thread which
-    // is compiling.  We just install a dummy exception object
-    thread->set_pending_exception(Universe::vm_exception(), file, line);
-    return true;
-  }
-  return false;
-}
-
 // This method should only be called from generated code,
 // therefore the exception oop should be in the oopmap.
 void Exceptions::_throw_oop(JavaThread* thread, const char* file, int line, oop exception) {
@@ -187,7 +186,7 @@ void Exceptions::_throw(JavaThread* thread, const char* file, int line, Handle h
 void Exceptions::_throw_msg(JavaThread* thread, const char* file, int line, Symbol* name, const char* message,
                             Handle h_loader, Handle h_protection_domain) {
   // Check for special boot-strapping/compiler-thread handling
-  if (special_exception(thread, file, line, name, message)) return;
+  if (special_exception(thread, file, line, Handle(), name, message)) return;
   // Create and throw exception
   Handle h_cause(thread, nullptr);
   Handle h_exception = new_exception(thread, name, message, h_cause, h_loader, h_protection_domain);
@@ -197,7 +196,7 @@ void Exceptions::_throw_msg(JavaThread* thread, const char* file, int line, Symb
 void Exceptions::_throw_msg_cause(JavaThread* thread, const char* file, int line, Symbol* name, const char* message, Handle h_cause,
                                   Handle h_loader, Handle h_protection_domain) {
   // Check for special boot-strapping/compiler-thread handling
-  if (special_exception(thread, file, line, name, message)) return;
+  if (special_exception(thread, file, line, Handle(), name, message)) return;
   // Create and throw exception and init cause
   Handle h_exception = new_exception(thread, name, message, h_cause, h_loader, h_protection_domain);
   _throw(thread, file, line, h_exception, message);
@@ -214,7 +213,7 @@ void Exceptions::_throw_cause(JavaThread* thread, const char* file, int line, Sy
 
 void Exceptions::_throw_args(JavaThread* thread, const char* file, int line, Symbol* name, Symbol* signature, JavaCallArguments *args) {
   // Check for special boot-strapping/compiler-thread handling
-  if (special_exception(thread, file, line, name, nullptr)) return;
+  if (special_exception(thread, file, line, Handle(), name, nullptr)) return;
   // Create and throw exception
   Handle h_loader(thread, nullptr);
   Handle h_prot(thread, nullptr);
@@ -434,6 +433,7 @@ void Exceptions::wrap_dynamic_exception(bool is_indy, JavaThread* THREAD) {
       // Pass through an Error, including BootstrapMethodError, any other form
       // of linkage error, or say OutOfMemoryError
       if (ls != nullptr) {
+        ResourceMark rm(THREAD);
         ls->print_cr("bootstrap method invocation wraps BSME around " PTR_FORMAT, p2i(exception));
         exception->print_on(ls);
       }
@@ -442,6 +442,7 @@ void Exceptions::wrap_dynamic_exception(bool is_indy, JavaThread* THREAD) {
 
     // Otherwise wrap the exception in a BootstrapMethodError
     if (ls != nullptr) {
+      ResourceMark rm(THREAD);
       ls->print_cr("%s throws BSME for " PTR_FORMAT, is_indy ? "invokedynamic" : "dynamic constant", p2i(exception));
       exception->print_on(ls);
     }
@@ -469,7 +470,7 @@ void Exceptions::count_out_of_memory_exceptions(Handle exception) {
   }
 }
 
-void print_oom_count(outputStream* st, const char *err, int count) {
+static void print_oom_count(outputStream* st, const char *err, int count) {
   if (count > 0) {
     st->print_cr("OutOfMemoryError %s=%d", err, count);
   }
@@ -565,11 +566,11 @@ void Exceptions::debug_check_abort_helper(Handle exception, const char* message)
 // for logging exceptions
 void Exceptions::log_exception(Handle exception, const char* message) {
   ResourceMark rm;
-  Symbol* detail_message = java_lang_Throwable::detail_message(exception());
+  const char* detail_message = java_lang_Throwable::message_as_utf8(exception());
   if (detail_message != nullptr) {
     log_info(exceptions)("Exception <%s: %s>\n thrown in %s",
                          exception->print_value_string(),
-                         detail_message->as_C_string(),
+                         detail_message,
                          message);
   } else {
     log_info(exceptions)("Exception <%s>\n thrown in %s",
