@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -79,6 +79,11 @@ module java.base {
     exports java.io;
     exports java.lang;
     exports java.lang.annotation;
+    exports java.lang.classfile;
+    exports java.lang.classfile.attribute;
+    exports java.lang.classfile.components;
+    exports java.lang.classfile.constantpool;
+    exports java.lang.classfile.instruction;
     exports java.lang.constant;
     exports java.lang.foreign;
     exports java.lang.invoke;
@@ -148,9 +153,14 @@ module java.base {
     // module declaration be annotated with jdk.internal.javac.ParticipatesInPreview
     exports jdk.internal.javac to
         java.compiler,
+        java.desktop, // for ScopedValue
         jdk.compiler,
         jdk.incubator.vector, // participates in preview features
-        jdk.jshell;
+        jdk.jartool, // participates in preview features
+        jdk.jdeps, // participates in preview features
+        jdk.jfr, // participates in preview features
+        jdk.jlink,   // participates in preview features
+        jdk.jshell; // participates in preview features
     exports jdk.internal.access to
         java.desktop,
         java.logging,
@@ -184,23 +194,6 @@ module java.base {
         jdk.jlink;
     exports jdk.internal.logger to
         java.logging;
-    exports jdk.internal.classfile to
-        jdk.jartool,
-        jdk.jdeps,
-        jdk.jlink,
-        jdk.jshell;
-    exports jdk.internal.classfile.attribute to
-        jdk.jartool,
-        jdk.jdeps,
-        jdk.jlink;
-    exports jdk.internal.classfile.constantpool to
-        jdk.jartool,
-        jdk.jdeps,
-        jdk.jlink;
-    exports jdk.internal.classfile.instruction to
-        jdk.jdeps,
-        jdk.jlink,
-        jdk.jshell;
     exports jdk.internal.org.objectweb.asm to
         jdk.jfr;
     exports jdk.internal.org.objectweb.asm.tree to
@@ -235,6 +228,7 @@ module java.base {
         java.instrument,
         java.management.rmi,
         jdk.jartool,
+        jdk.compiler,
         jdk.jfr,
         jdk.jlink,
         jdk.jpackage;
@@ -274,8 +268,6 @@ module java.base {
         jdk.jfr;
     exports jdk.internal.util.xml.impl to
         jdk.jfr;
-    exports jdk.internal.util.random to
-        jdk.random;
     exports jdk.internal.util to
         java.desktop,
         java.prefs,
@@ -402,7 +394,6 @@ module java.base {
     uses java.time.chrono.AbstractChronology;
     uses java.time.chrono.Chronology;
     uses java.time.zone.ZoneRulesProvider;
-    uses java.util.random.RandomGenerator;
     uses java.util.spi.CalendarDataProvider;
     uses java.util.spi.CalendarNameProvider;
     uses java.util.spi.CurrencyNameProvider;
@@ -428,9 +419,4 @@ module java.base {
     provides java.nio.file.spi.FileSystemProvider with
         jdk.internal.jrtfs.JrtFileSystemProvider;
 
-    provides java.util.random.RandomGenerator with
-        java.security.SecureRandom,
-        java.util.Random,
-        java.util.SplittableRandom;
-
 }
