@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -36,6 +36,7 @@ import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.net.HttpURLConnection;
+import java.net.MalformedURLException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
@@ -79,6 +80,7 @@ import javax.print.attribute.standard.MediaSizeName;
 import javax.print.attribute.standard.MediaTray;
 import javax.print.attribute.standard.NumberUp;
 import javax.print.attribute.standard.OrientationRequested;
+import javax.print.attribute.standard.OutputBin;
 import javax.print.attribute.standard.PDLOverrideSupported;
 import javax.print.attribute.standard.PageRanges;
 import javax.print.attribute.standard.PagesPerMinute;
@@ -117,6 +119,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
     private static final String FORCE_PIPE_PROP = "sun.print.ippdebug";
 
     static {
+        @SuppressWarnings("removal")
         String debugStr = java.security.AccessController.doPrivileged(
                   new sun.security.action.GetPropertyAction(FORCE_PIPE_PROP));
 
@@ -136,6 +139,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
     private DocFlavor[] supportedDocFlavors;
     private Class<?>[] supportedCats;
     private MediaTray[] mediaTrays;
+    private OutputBin[] outputBins;
     private MediaSizeName[] mediaSizeNames;
     private CustomMediaSizeName[] customMediaSizeNames;
     private int defaultMediaIndex;
@@ -209,6 +213,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
         new RequestingUserName("", Locale.getDefault()),
         //SheetCollate.UNCOLLATED, //CUPS has no sheet collate?
         Sides.ONE_SIDED,
+        OutputBin.TOP,
     };
 
 
@@ -405,7 +410,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
         defaultMediaIndex = -1;
         try {
             myURL =
-                new URL(uriStr.replaceFirst("ipp", "http"));
+                newURL(uriStr.replaceFirst("ipp", "http"));
         } catch (Exception e) {
             IPPPrintService.debug_println(debugPrefix+
                                           " IPPPrintService, myURL="+
@@ -438,6 +443,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
             if ((urlConnection = getIPPConnection(myURL)) == null) {
                 mediaSizeNames = new MediaSizeName[0];
                 mediaTrays = new MediaTray[0];
+                outputBins = new OutputBin[0];
                 debug_println(debugPrefix+"initAttributes, NULL urlConnection ");
                 init = true;
                 return;
@@ -454,12 +460,17 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                 // IPP currently does not support it but PPD does.
 
                 try {
-                    cps = new CUPSPrinter(printer);
-                    mediaSizeNames = cps.getMediaSizeNames();
-                    mediaTrays = cps.getMediaTrays();
-                    customMediaSizeNames = cps.getCustomMediaSizeNames();
-                    defaultMediaIndex = cps.getDefaultMediaIndex();
-                    rawResolutions = cps.getRawResolutions();
+                    if (cps == null) {
+                        cps = new CUPSPrinter(printer);
+                        mediaSizeNames = cps.getMediaSizeNames();
+                        mediaTrays = cps.getMediaTrays();
+                        outputBins = PrintServiceLookupProvider.isMac()
+                                ? cps.getOutputBins()
+                                : getSupportedOutputBins();
+                        customMediaSizeNames = cps.getCustomMediaSizeNames();
+                        defaultMediaIndex = cps.getDefaultMediaIndex();
+                        rawResolutions = cps.getRawResolutions();
+                    }
                     urlConnection.disconnect();
                     init = true;
                     return;
@@ -489,6 +500,11 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                 mediaTrays = new MediaTray[trayList.size()];
                 mediaTrays = trayList.toArray(mediaTrays);
             }
+
+            if (outputBins == null) {
+                outputBins = getSupportedOutputBins();
+            }
+
             urlConnection.disconnect();
 
             init = true;
@@ -497,6 +513,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
 
 
     public DocPrintJob createPrintJob() {
+        @SuppressWarnings("removal")
         SecurityManager security = System.getSecurityManager();
         if (security != null) {
             security.checkPrintJobAccess();
@@ -601,6 +618,9 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                         Finishings[] fAll = (Finishings[])
                             (new ExtFinishing(100)).getAll();
                         for (int j=0; j<fAll.length; j++) {
+                            if (fAll[j] == null) {
+                                continue;
+                            }
                             if (finArray[i] == fAll[j].getValue()) {
                                 finSup[i] = fAll[j];
                                 break;
@@ -613,7 +633,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
         } else if (category == JobName.class) {
             return new JobName("Java Printing", null);
         } else if (category == JobSheets.class) {
-            JobSheets arr[] = new JobSheets[2];
+            JobSheets[] arr = new JobSheets[2];
             arr[0] = JobSheets.NONE;
             arr[1] = JobSheets.STANDARD;
             return arr;
@@ -666,8 +686,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
 
             int match = -1;
             Media media = (Media)attributes.get(Media.class);
-            if (media != null && media instanceof MediaSizeName) {
-                MediaSizeName msn = (MediaSizeName)media;
+            if (media instanceof MediaSizeName msn) {
 
                 // case when no supported mediasizenames are reported
                 // check given media against the default
@@ -820,6 +839,8 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                 new PrinterResolution[supportedRes.length];
             System.arraycopy(supportedRes, 0, arr, 0, supportedRes.length);
             return arr;
+        } else if (category == OutputBin.class) {
+            return Arrays.copyOf(outputBins, outputBins.length);
         }
 
         return null;
@@ -827,7 +848,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
 
     //This class is for getting all pre-defined Finishings
     @SuppressWarnings("serial") // JDK implementation class
-    private class ExtFinishing extends Finishings {
+    private static class ExtFinishing extends Finishings {
         ExtFinishing(int value) {
             super(100); // 100 to avoid any conflicts with predefined values.
         }
@@ -922,7 +943,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                                 gifImagesAdded = true;
                             } else if (mimeType.equals("image/jpeg")) {
                                 jpgImagesAdded = true;
-                            } else if (mimeType.indexOf("postscript") != -1) {
+                            } else if (mimeType.contains("postscript")) {
                                 psSupported = true;
                             }
                             break;
@@ -1006,15 +1027,15 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
      * Returns the matching standard Media using string comparison of names.
      */
     private Media getIPPMedia(String mediaName) {
-        CustomMediaSizeName sampleSize = new CustomMediaSizeName("sample", "",
-                                                                 0, 0);
+        CustomMediaSizeName sampleSize =
+                CustomMediaSizeName.create("sample", "", 0, 0);
         Media[] sizes = sampleSize.getSuperEnumTable();
         for (int i=0; i<sizes.length; i++) {
             if (mediaName.equals(""+sizes[i])) {
                 return sizes[i];
             }
         }
-        CustomMediaTray sampleTray = new CustomMediaTray("sample", "");
+        CustomMediaTray sampleTray = CustomMediaTray.create("sample", "");
         Media[] trays = sampleTray.getSuperEnumTable();
         for (int i=0; i<trays.length; i++) {
             if (mediaName.equals(""+trays[i])) {
@@ -1046,6 +1067,25 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
         return new Media[0];
     }
 
+    private OutputBin[] getSupportedOutputBins() {
+        if ((getAttMap != null) && getAttMap.containsKey("output-bin-supported")) {
+
+            AttributeClass attribClass = getAttMap.get("output-bin-supported");
+
+            if (attribClass != null) {
+                String[] values = attribClass.getArrayOfStringValues();
+                if (values == null || values.length == 0) {
+                    return null;
+                }
+                OutputBin[] outputBinNames = new OutputBin[values.length];
+                for (int i = 0; i < values.length; i++) {
+                    outputBinNames[i] = CustomOutputBin.createOutputBin(values[i], values[i]);
+                }
+                return outputBinNames;
+            }
+        }
+        return null;
+    }
 
     public synchronized Class<?>[] getSupportedAttributeCategories() {
         if (supportedCats != null) {
@@ -1063,6 +1103,11 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                 (PrintRequestAttribute)printReqAttribDefault[i];
             if (getAttMap != null &&
                 getAttMap.containsKey(pra.getName()+"-supported")) {
+
+                if (pra == OutputBin.TOP && (outputBins == null || outputBins.length == 0)) {
+                    continue;
+                }
+
                 catList.add(pra.getCategory());
             }
         }
@@ -1141,6 +1186,11 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
             return true;
         }
 
+        if (category == OutputBin.class
+                && (outputBins == null || outputBins.length == 0)) {
+            return false;
+        }
+
         for (int i=0;i<supportedCats.length;i++) {
             if (category == supportedCats[i]) {
                 return true;
@@ -1220,9 +1270,17 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
 
 
     public synchronized PrintServiceAttributeSet getAttributes() {
-        // update getAttMap by sending again get-attributes IPP request
-        init = false;
-        initAttributes();
+        if (!init) {
+            // get all attributes for the first time.
+            initAttributes();
+        } else {
+            // only need service attributes updated.
+            // update getAttMap by sending again get-attributes IPP request
+            if ((urlConnection = getIPPConnection(myURL)) != null) {
+                opGetAttributes();
+                urlConnection.disconnect();
+            }
+        }
 
         HashPrintServiceAttributeSet attrs =
             new HashPrintServiceAttributeSet();
@@ -1378,7 +1436,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                 flavor.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                 URI uri = ((Destination)attr).getURI();
                 if ("file".equals(uri.getScheme()) &&
-                    !(uri.getSchemeSpecificPart().equals(""))) {
+                    !uri.getSchemeSpecificPart().isEmpty()) {
                     return true;
                 }
             }
@@ -1463,6 +1521,18 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                DialogTypeSelection dst = (DialogTypeSelection)attr;
                return attr == DialogTypeSelection.COMMON;
             }
+        } else if (attr.getCategory() == OutputBin.class) {
+            if (attr instanceof CustomOutputBin) {
+                return true;
+            }
+            String name = attr.toString();
+            for (OutputBin outputBin : outputBins) {
+                String choice = ((CustomOutputBin) outputBin).getChoiceName();
+                if (name.equalsIgnoreCase(choice) || name.replaceAll("-", "").equalsIgnoreCase(choice)) {
+                    return true;
+                }
+            }
+            return false;
         }
         return true;
     }
@@ -1535,7 +1605,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
             if (mediaSizeNames.length == 0) {
                 String defaultCountry = Locale.getDefault().getCountry();
                 if (defaultCountry != null &&
-                    (defaultCountry.equals("") ||
+                    (defaultCountry.isEmpty() ||
                      defaultCountry.equals(Locale.US.getCountry()) ||
                      defaultCountry.equals(Locale.CANADA.getCountry()))) {
                     return MediaSizeName.NA_LETTER;
@@ -1550,7 +1620,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                     return mediaSizeNames[defaultMediaIndex];
                 } else {
                     for (int i=0; i< mediaSizeNames.length; i++) {
-                        if (mediaSizeNames[i].toString().indexOf(name) != -1) {
+                        if (mediaSizeNames[i].toString().contains(name)) {
                             defaultMediaIndex = i;
                             return mediaSizeNames[defaultMediaIndex];
                         }
@@ -1572,7 +1642,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                  String defaultCountry = Locale.getDefault().getCountry();
                  float iw, ih;
                  if (defaultCountry != null &&
-                     (defaultCountry.equals("") ||
+                     (defaultCountry.isEmpty() ||
                       defaultCountry.equals(Locale.US.getCountry()) ||
                       defaultCountry.equals(Locale.CANADA.getCountry()))) {
                      iw = MediaSize.NA.LETTER.getX(Size2DSyntax.INCH) - 0.5f;
@@ -1631,6 +1701,10 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
              } else {
                  return new PrinterResolution(300, 300, PrinterResolution.DPI);
              }
+        } else if (category == OutputBin.class) {
+            if (attribClass != null) {
+                return CustomOutputBin.createOutputBin(attribClass.getStringValue(), attribClass.getStringValue());
+            }
         }
 
         return null;
@@ -1758,7 +1832,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
             if (isCupsPrinter) {
                 try {
                     urlConnection = getIPPConnection(
-                                             new URL(myURL+".ppd"));
+                                             newURL(myURL+".ppd"));
 
                    InputStream is = urlConnection.getInputStream();
                    if (is != null) {
@@ -1789,17 +1863,18 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
         try {
             debug_println(debugPrefix+"opGetAttributes myURI "+myURI+" myURL "+myURL);
 
-            AttributeClass attClNoUri[] = {
+            AttributeClass[] attClNoUri = {
                 AttributeClass.ATTRIBUTES_CHARSET,
                 AttributeClass.ATTRIBUTES_NATURAL_LANGUAGE};
 
-            AttributeClass attCl[] = {
+            AttributeClass[] attCl = {
                 AttributeClass.ATTRIBUTES_CHARSET,
                 AttributeClass.ATTRIBUTES_NATURAL_LANGUAGE,
                 new AttributeClass("printer-uri",
                                    AttributeClass.TAG_URI,
                                    ""+myURI)};
 
+            @SuppressWarnings("removal")
             OutputStream os = java.security.AccessController.
                 doPrivileged(new java.security.PrivilegedAction<OutputStream>() {
                     public OutputStream run() {
@@ -1909,7 +1984,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
             return null;
         }
 
-        byte response[] = new byte[MAX_ATTRIBUTE_LENGTH];
+        byte[] response = new byte[MAX_ATTRIBUTE_LENGTH];
         try {
 
             DataInputStream ois = new DataInputStream(inputStream);
@@ -1956,7 +2031,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                             outObj.write(counter);
                             outObj.flush();
                             outObj.close();
-                            byte outArray[] = outObj.toByteArray();
+                            byte[] outArray = outObj.toByteArray();
 
                             // if key exists, new HashMap
                             if (responseMap.containsKey(attribStr)) {
@@ -2020,7 +2095,7 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
                             responseMap = new HashMap<>();
                         }
 
-                        byte outArray[] = outObj.toByteArray();
+                        byte[] outArray = outObj.toByteArray();
 
                         AttributeClass ac =
                             new AttributeClass(attribStr,
@@ -2071,4 +2146,9 @@ public class IPPPrintService implements PrintService, SunPrinterJobService {
     public int hashCode() {
         return this.getClass().hashCode()+getName().hashCode();
     }
+
+    @SuppressWarnings("deprecation")
+    private static URL newURL(String spec) throws MalformedURLException {
+        return new URL(spec);
+    }
 }
