@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -222,8 +222,6 @@ util_initialize(JNIEnv *env)
                     "<init>", "(Ljava/lang/ThreadGroup;Ljava/lang/String;)V");
         gdata->threadSetDaemon =
                 getMethod(env, gdata->threadClass, "setDaemon", "(Z)V");
-        gdata->threadResume =
-                getMethod(env, gdata->threadClass, "resume", "()V");
         gdata->systemGetProperty =
                 getStaticMethod(env, gdata->systemClass,
                     "getProperty", "(Ljava/lang/String;)Ljava/lang/String;");
@@ -245,6 +243,7 @@ util_initialize(JNIEnv *env)
         }
         localSystemThreadGroup = groups[0];
         saveGlobalRef(env, localSystemThreadGroup, &(gdata->systemThreadGroup));
+        jvmtiDeallocate(groups);
 
         /* Get some basic Java property values we will need at some point */
         gdata->property_java_version
@@ -1040,7 +1039,7 @@ debugMonitorWait(jrawMonitorID monitor)
      * According to the JLS (17.8), here we have
      * either :
      * a- been notified
-     * b- gotten a suprious wakeup
+     * b- gotten a spurious wakeup
      * c- been interrupted
      * If both a and c have happened, the VM must choose
      * which way to return - a or c.  If it chooses c
@@ -1074,23 +1073,6 @@ debugMonitorWait(jrawMonitorID monitor)
     }
 }
 
-void
-debugMonitorTimedWait(jrawMonitorID monitor, jlong millis)
-{
-    jvmtiError error;
-    error = JVMTI_FUNC_PTR(gdata->jvmti,RawMonitorWait)
-        (gdata->jvmti, monitor, millis);
-    if (error == JVMTI_ERROR_INTERRUPT) {
-        /* See comment above */
-        handleInterrupt();
-        error = JVMTI_ERROR_NONE;
-    }
-    error = ignore_vm_death(error);
-    if (error != JVMTI_ERROR_NONE) {
-        EXIT_ERROR(error, "on raw monitor timed wait");
-    }
-}
-
 void
 debugMonitorNotify(jrawMonitorID monitor)
 {
@@ -1548,6 +1530,13 @@ isClass(jobject object)
     return JNI_FUNC_PTR(env,IsInstanceOf)(env, object, gdata->classClass);
 }
 
+jboolean
+isVThread(jobject object)
+{
+    JNIEnv *env = getEnv();
+    return JNI_FUNC_PTR(env,IsVirtualThread)(env, object);
+}
+
 jboolean
 isThread(jobject object)
 {
@@ -1729,7 +1718,7 @@ getSpecialJvmti(void)
     jvmtiCapabilities caps;
 
     rc = JVM_FUNC_PTR(gdata->jvm,GetEnv)
-                     (gdata->jvm, (void **)&jvmti, JVMTI_VERSION_1);
+                     (gdata->jvm, (void **)&jvmti, JVMTI_VERSION);
     if (rc != JNI_OK) {
         return NULL;
     }
@@ -1928,7 +1917,7 @@ eventIndexInit(void)
     index2jvmti[EI_THREAD_START       -EI_min] = JVMTI_EVENT_THREAD_START;
     index2jvmti[EI_THREAD_END         -EI_min] = JVMTI_EVENT_THREAD_END;
     index2jvmti[EI_CLASS_PREPARE      -EI_min] = JVMTI_EVENT_CLASS_PREPARE;
-    index2jvmti[EI_GC_FINISH          -EI_min] = JVMTI_EVENT_GARBAGE_COLLECTION_FINISH;
+    index2jvmti[EI_CLASS_UNLOAD       -EI_min] = 0; // No mapping to JVMTI event
     index2jvmti[EI_CLASS_LOAD         -EI_min] = JVMTI_EVENT_CLASS_LOAD;
     index2jvmti[EI_FIELD_ACCESS       -EI_min] = JVMTI_EVENT_FIELD_ACCESS;
     index2jvmti[EI_FIELD_MODIFICATION -EI_min] = JVMTI_EVENT_FIELD_MODIFICATION;
@@ -1941,6 +1930,8 @@ eventIndexInit(void)
     index2jvmti[EI_MONITOR_WAITED     -EI_min] = JVMTI_EVENT_MONITOR_WAITED;
     index2jvmti[EI_VM_INIT            -EI_min] = JVMTI_EVENT_VM_INIT;
     index2jvmti[EI_VM_DEATH           -EI_min] = JVMTI_EVENT_VM_DEATH;
+    index2jvmti[EI_VIRTUAL_THREAD_START -EI_min] = JVMTI_EVENT_VIRTUAL_THREAD_START;
+    index2jvmti[EI_VIRTUAL_THREAD_END   -EI_min] = JVMTI_EVENT_VIRTUAL_THREAD_END;
 
     index2jdwp[EI_SINGLE_STEP         -EI_min] = JDWP_EVENT(SINGLE_STEP);
     index2jdwp[EI_BREAKPOINT          -EI_min] = JDWP_EVENT(BREAKPOINT);
@@ -1949,7 +1940,7 @@ eventIndexInit(void)
     index2jdwp[EI_THREAD_START        -EI_min] = JDWP_EVENT(THREAD_START);
     index2jdwp[EI_THREAD_END          -EI_min] = JDWP_EVENT(THREAD_END);
     index2jdwp[EI_CLASS_PREPARE       -EI_min] = JDWP_EVENT(CLASS_PREPARE);
-    index2jdwp[EI_GC_FINISH           -EI_min] = JDWP_EVENT(CLASS_UNLOAD);
+    index2jdwp[EI_CLASS_UNLOAD        -EI_min] = JDWP_EVENT(CLASS_UNLOAD);
     index2jdwp[EI_CLASS_LOAD          -EI_min] = JDWP_EVENT(CLASS_LOAD);
     index2jdwp[EI_FIELD_ACCESS        -EI_min] = JDWP_EVENT(FIELD_ACCESS);
     index2jdwp[EI_FIELD_MODIFICATION  -EI_min] = JDWP_EVENT(FIELD_MODIFICATION);
@@ -1962,24 +1953,35 @@ eventIndexInit(void)
     index2jdwp[EI_MONITOR_WAITED      -EI_min] = JDWP_EVENT(MONITOR_WAITED);
     index2jdwp[EI_VM_INIT             -EI_min] = JDWP_EVENT(VM_INIT);
     index2jdwp[EI_VM_DEATH            -EI_min] = JDWP_EVENT(VM_DEATH);
+    /* Just map VIRTUAL_THREAD_START/END to THREAD_START/END. */
+    index2jdwp[EI_VIRTUAL_THREAD_START -EI_min] = JDWP_EVENT(THREAD_START);
+    index2jdwp[EI_VIRTUAL_THREAD_END   -EI_min] = JDWP_EVENT(THREAD_END);
 }
 
 jdwpEvent
-eventIndex2jdwp(EventIndex i)
+eventIndex2jdwp(EventIndex ei)
 {
-    if ( i < EI_min || i > EI_max ) {
-        EXIT_ERROR(AGENT_ERROR_INVALID_INDEX,"bad EventIndex");
+    jdwpEvent event = 0;
+    if (ei >= EI_min && ei <= EI_max) {
+        event = index2jdwp[ei - EI_min];
     }
-    return index2jdwp[i-EI_min];
+    if (event == 0) {
+        EXIT_ERROR(AGENT_ERROR_INVALID_INDEX, "bad EventIndex");
+    }
+    return event;
 }
 
 jvmtiEvent
-eventIndex2jvmti(EventIndex i)
+eventIndex2jvmti(EventIndex ei)
 {
-    if ( i < EI_min || i > EI_max ) {
-        EXIT_ERROR(AGENT_ERROR_INVALID_INDEX,"bad EventIndex");
+    jvmtiEvent event = 0;
+    if (ei >= EI_min && ei <= EI_max) {
+        event = index2jvmti[ei - EI_min];
     }
-    return index2jvmti[i-EI_min];
+    if (event == 0) {
+        EXIT_ERROR(AGENT_ERROR_INVALID_INDEX, "bad EventIndex");
+    }
+    return event;
 }
 
 #ifdef DEBUG
@@ -2002,8 +2004,8 @@ eventIndex2EventName(EventIndex ei)
             return "EI_THREAD_END";
         case EI_CLASS_PREPARE:
             return "EI_CLASS_PREPARE";
-        case EI_GC_FINISH:
-            return "EI_GC_FINISH";
+        case EI_CLASS_UNLOAD:
+            return "EI_CLASS_UNLOAD";
         case EI_CLASS_LOAD:
             return "EI_CLASS_LOAD";
         case EI_FIELD_ACCESS:
@@ -2028,6 +2030,10 @@ eventIndex2EventName(EventIndex ei)
             return "EI_VM_INIT";
         case EI_VM_DEATH:
             return "EI_VM_DEATH";
+        case EI_VIRTUAL_THREAD_START:
+            return "EI_VIRTUAL_THREAD_START";
+        case EI_VIRTUAL_THREAD_END:
+            return "EI_VIRTUAL_THREAD_END";
         default:
             JDI_ASSERT(JNI_FALSE);
             return "Bad EI";
@@ -2055,7 +2061,7 @@ jdwp2EventIndex(jdwpEvent eventType)
         case JDWP_EVENT(CLASS_PREPARE):
             return EI_CLASS_PREPARE;
         case JDWP_EVENT(CLASS_UNLOAD):
-            return EI_GC_FINISH;
+            return EI_CLASS_UNLOAD;
         case JDWP_EVENT(CLASS_LOAD):
             return EI_CLASS_LOAD;
         case JDWP_EVENT(FIELD_ACCESS):
@@ -2111,8 +2117,6 @@ jvmti2EventIndex(jvmtiEvent kind)
             return EI_THREAD_END;
         case JVMTI_EVENT_CLASS_PREPARE:
             return EI_CLASS_PREPARE;
-        case JVMTI_EVENT_GARBAGE_COLLECTION_FINISH:
-            return EI_GC_FINISH;
         case JVMTI_EVENT_CLASS_LOAD:
             return EI_CLASS_LOAD;
         case JVMTI_EVENT_FIELD_ACCESS:
@@ -2141,6 +2145,12 @@ jvmti2EventIndex(jvmtiEvent kind)
             return EI_VM_INIT;
         case JVMTI_EVENT_VM_DEATH:
             return EI_VM_DEATH;
+        /* vthread events */
+        case JVMTI_EVENT_VIRTUAL_THREAD_START:
+            return EI_VIRTUAL_THREAD_START;
+        case JVMTI_EVENT_VIRTUAL_THREAD_END:
+            return EI_VIRTUAL_THREAD_END;
+
         default:
             EXIT_ERROR(AGENT_ERROR_INVALID_INDEX,"JVMTI to EventIndex mapping");
             break;
@@ -2254,6 +2264,8 @@ map2jdwpError(jvmtiError error)
             return JDWP_ERROR(METHOD_MODIFIERS_CHANGE_NOT_IMPLEMENTED);
         case JVMTI_ERROR_UNSUPPORTED_REDEFINITION_CLASS_ATTRIBUTE_CHANGED:
             return JDWP_ERROR(CLASS_ATTRIBUTE_CHANGE_NOT_IMPLEMENTED);
+        case JVMTI_ERROR_UNSUPPORTED_OPERATION:
+            return JDWP_ERROR(NOT_IMPLEMENTED);
         case AGENT_ERROR_NOT_CURRENT_FRAME:
             return JDWP_ERROR(NOT_CURRENT_FRAME);
         case AGENT_ERROR_INVALID_TAG:
@@ -2409,7 +2421,7 @@ log_debugee_location(const char *func,
         }
 
         /* Issue log message */
-        LOG_LOC(("%s: debugee: thread=%p(%s:0x%x),method=%p(%s@%d;%s)",
+        LOG_LOC(("%s: debuggee: thread=%p(%s:0x%x),method=%p(%s@%d;%s)",
                 func,
                 thread, info.name==NULL ? "?" : info.name, state,
                 method, method_name==NULL ? "?" : method_name,
